package com.azt.itower.command;

/**
 *
 * This command class executes for every execution of a card.
 * It can easily support undo facility
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */

import com.azt.itower.game.*;
import com.azt.itower.util.*;

public class ChangeAttributeCommand extends AbstractCommand {
    public static final int CURRENT = 0;
    public static final int BOTH = 2;
    public static final int OTHER = 1;
    private String fTypeA;
    private int fChangeA;
    private String fTypeB;
    private int fChangeB;
    private int fTarget;

    /**
     * The constructor of a command needs the type of the targetting change,
     * the change, and the target player. For example the type can be "Wall", the
     * change can be -15 and the target OTHER_PLAYER
     * @param String The type of targetting change
     * @param int The signed amount of change ( + / - )
     * @param int The target ( .SAME .OTHER etc)
     */
    public ChangeAttributeCommand(String typeA, int changeA, int target) {
        fTypeA = typeA;
        fChangeA = changeA;
        fTarget = target;
    }

    public String getType() {
      return fTypeA;
    }

    public int getChange() {
      return fChangeA;
    }

    public int getTarget() {
     return fTarget;
    }

    /**
     * Executes the command, by changing the state of the player
     */
    public void execute() {
        Player player = null;
        if (fTarget == CURRENT) {
            player = IvoryTower.currentPlayer;
        } else if (fTarget == OTHER) {
            player = IvoryTower.otherPlayer;
        } else if (fTarget == BOTH) {
            player = IvoryTower.currentPlayer;
        }
        if (fTypeA.equals("Gems")) {
            player.setGems(player.getGems() + fChangeA);
            if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISEGEMS_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYGEMS_SND);

        }
        else if (fTypeA.equals("Monsters")) {
            player.setMonsters(player.getMonsters() + fChangeA);
            if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISEMONSTERS_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYMONSTERS_SND);

        }
        else if (fTypeA.equals("Stones")) {
            player.setStones(player.getStones() + fChangeA);
            if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISESTONES_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYSTONES_SND);

        }
        else if (fTypeA.equals("StoneProduction")) {
            player.setStoneProduction(player.getStoneProduction() + fChangeA);
             if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISESTONEPROD_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYSTONEPROD_SND);
        }
        else if (fTypeA.equals("GemProduction")) {
            player.setGemProduction(player.getGemProduction() + fChangeA);
             if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISEGEMPROD_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYGEMPROD_SND);

        }
        else if (fTypeA.equals("MonsterProduction")) {
            player.setMonsterProduction(player.getMonsterProduction() + fChangeA);
             if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kRAISEMONSTERPROD_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYMONSTERPROD_SND);
        }
        else if (fTypeA.equals("Wall")) {

            if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kBUILDWALL_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYWALL_SND);

            player.setWallSize(player.getWallSize() + fChangeA);
        }
        else if (fTypeA.equals("Tower")) {

            if (fChangeA>0)
              SoundRegistry.getInstance().playWavAudio(kBUILDCASTLE_SND);
            else
              SoundRegistry.getInstance().playWavAudio(kDESTROYCASTLE_SND);

            player.setTowerSize(player.getTowerSize() + fChangeA);

        }
    }

    /**
     * Paints the resources data of the first player
     */
     public void unexecute() {
     }

}
