package com.azt.itower.deck;

import java.awt.Point;
import com.azt.itower.game.GameException;
import com.azt.itower.game.CardModel;
import com.azt.itower.command.*;

/**
 * Title: The Ivory Tower Card Game <p> Description:  A first attempt at a fantasy card game!! <p> Copyright feb2000 AZT  Copyright (c) <p>
 * A Factory Class that adheres to the AbstractDeckFactory rules and provides
 * all the cards of the purple deck
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public class PurpleDeckFactory extends AbstractDeckFactory {

   /**
     * Inititalised The GreyDeckFactory
     * @exception GameException
     */
    public void init() throws GameException {
        init(kPURPLE_DECK_IMG, CardModel.kGEMS);
    }

    /**
     * Creates a model for a card belonging to particular point (row, column)
     * of that deck. The massive switch statement is responsible for creating
     * each card of the purple deck.
     * @param Point The point that defines the column and row of the card
     * @return CardModel The CardModel that every Card needs to get state and behaviour
     */
     protected CardModel createCardModel(Point p) {
        CardModel model = null;
        switch (p.y) {
            case 0:
                switch (p.x) {
                    case 0:
                        model = new CardModel("High Morale", 0, getCardModelType(), "Acquire +5 Gems",
                            new ChangeAttributeCommand("Gems", 5, ChangeAttributeCommand.CURRENT));
                        break;
                    case 1:
                        model = new CardModel("Gandalf", 0, getCardModelType(), " Acquire +7 Gems",
                            new ChangeAttributeCommand("Gems", 7, ChangeAttributeCommand.CURRENT));
                        break;
                    case 2:
                        model = new CardModel("Mine Portal", 12, getCardModelType(), " Raise +1 Stone Prod.",
                            new ChangeAttributeCommand("StoneProduction", 1, ChangeAttributeCommand.CURRENT));
                        break;
                    case 3:
                        model = new CardModel("Meteor Fall", 25, getCardModelType(), " Attack +15 Enemy Wall ",
                            new ChangeAttributeCommand("Wall", -15, ChangeAttributeCommand.OTHER));
                        break;
                    case 4:
                        model = new CardModel("Mass Bless", 8, getCardModelType(), " Build +8 Wall ",
                            new ChangeAttributeCommand("Wall", 8, ChangeAttributeCommand.CURRENT));
                        break;
                    case 5:
                        model = new CardModel("Fire Orb", 0, getCardModelType(), " Acquire +5 Gems",
                            new ChangeAttributeCommand("Gems", 5, ChangeAttributeCommand.CURRENT));
                        break;
                    case 6:
                        model = new CardModel("Air Weave", 10, getCardModelType(), " Attack +10 Enemy Wall ",
                            new ChangeAttributeCommand("Wall", -10, ChangeAttributeCommand.OTHER));
                        break;
                    case 7:
                        model = new CardModel("Armageddon", 45, getCardModelType(), " Attack +25 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -25, ChangeAttributeCommand.OTHER));
                        break;
                    case 8:
                        model = new CardModel("Dark Night", 11, getCardModelType(), " Destroy +2 Enemy Stone Prod. ",
                            new ChangeAttributeCommand("StoneProduction", -2, ChangeAttributeCommand.OTHER));
                        break;
                    case 9:
                        model = new CardModel("Castle Baths", 22, getCardModelType(), " Raise +1 Stone Prod. ",
                            new ChangeAttributeCommand("StoneProduction", 1, ChangeAttributeCommand.CURRENT));
                        break;
                }
                break;
            case 1:
                switch (p.x) {
                    case 0:
                        model = new CardModel("Gem Wizard", 3, getCardModelType(), " Acquire +10 Gems",
                            new ChangeAttributeCommand("Gems", 10, ChangeAttributeCommand.CURRENT));
                        break;
                    case 1:
                        model = new CardModel("Crystal Dragon", 15, getCardModelType(), " Build +9 Tower ",
                            new ChangeAttributeCommand("Tower", 9, ChangeAttributeCommand.CURRENT));
                        break;
                    case 2:
                        model = new CardModel("Hell Keeper", 10, getCardModelType(), " Attack +7 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -7, ChangeAttributeCommand.OTHER));
                        break;
                    case 3:
                        model = new CardModel("Hell Spawn", 4, getCardModelType(), " Build +4 Wall ",
                            new ChangeAttributeCommand("Wall", 4, ChangeAttributeCommand.CURRENT));
                        break;
                    case 4:
                        model = new CardModel("Wizdom", 15, getCardModelType(), " Acquire +10 Gems",
                            new ChangeAttributeCommand("Gems", 10, ChangeAttributeCommand.CURRENT));
                        break;
                    case 5:
                        model = new CardModel("Sulfur Dragon", 11, getCardModelType(), " Attack +11 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -11, ChangeAttributeCommand.OTHER));
                        break;
                    case 6:
                        model = new CardModel("Solymir", 11, getCardModelType(), " Attack +11 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -11, ChangeAttributeCommand.OTHER));
                        break;
                    case 7:
                        model = new CardModel("Graig Hack", 8, getCardModelType(), " Build +5 Tower ",
                            new ChangeAttributeCommand("Tower", 5, ChangeAttributeCommand.CURRENT));
                        break;
                    case 8:
                        model = new CardModel("Gem Sheer", 10, getCardModelType(), " Build +10 Tower ",
                            new ChangeAttributeCommand("Tower", 10, ChangeAttributeCommand.CURRENT));
                        break;
                    case 9:
                        model = new CardModel("Dragon Ride", 12, getCardModelType(), " Build +15 Tower ",
                            new ChangeAttributeCommand("Tower", 15, ChangeAttributeCommand.CURRENT));
                        break;
                }
                break;
            case 2:
                switch (p.x) {
                    case 0:
                        model = new CardModel("Lizzard Army", 0, getCardModelType(), " Acquire +12 Monsters",
                            new ChangeAttributeCommand("Monsters", 12, ChangeAttributeCommand.CURRENT));
                        break;
                    case 1:
                        model = new CardModel("Magic Tools", 2, getCardModelType(), " Raise +2 Monst. Prod.",
                            new ChangeAttributeCommand("MonsterProduction", 2, ChangeAttributeCommand.CURRENT));
                        break;
                    case 2:
                        model = new CardModel("Heretic", 2, getCardModelType(), " Build +14 Wall ",
                            new ChangeAttributeCommand("Wall", 14, ChangeAttributeCommand.CURRENT));
                        break;
                    case 3:
                        model = new CardModel("Empress", 10, getCardModelType(), " Raise +2 Gem Prod.",
                            new ChangeAttributeCommand("GemProduction", 2, ChangeAttributeCommand.CURRENT));
                        break;
                    case 4:
                        model = new CardModel("Blue Queen", 0, getCardModelType(), " Raise +2 Stone Prod.",
                            new ChangeAttributeCommand("StoneProduction", 2, ChangeAttributeCommand.CURRENT));
                        break;
                    case 5:
                        model = new CardModel("Sorcery", 0, getCardModelType(), " Acquire +12 Gems",
                            new ChangeAttributeCommand("Gems", 12, ChangeAttributeCommand.CURRENT));
                        break;
                    case 6:
                        model = new CardModel("Prophecy", 0, getCardModelType(), " Acquire +12 Stones",
                            new ChangeAttributeCommand("Stones", 12, ChangeAttributeCommand.CURRENT));
                        break;
                    case 7:
                        model = new CardModel("Devil Contract", 30, getCardModelType(), " Attack +17 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -17, ChangeAttributeCommand.OTHER));
                        break;
                    case 8:
                        model = new CardModel("Ring Lord", 12, getCardModelType(), " Attack +14 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -14, ChangeAttributeCommand.OTHER));
                        break;
                    case 9:
                        model = new CardModel("Wise Faery", 10, getCardModelType(), " Raise +1 Gem Prod.",
                            new ChangeAttributeCommand("GemProduction", 1, ChangeAttributeCommand.CURRENT));
                        break;
                }
                break;
            case 3:
                switch (p.x) {
                    case 0:
                        model = new CardModel("Absorb Mana", 10, getCardModelType(), " Destroy +10 Gems ",
                            new ChangeAttributeCommand("Gems", -10, ChangeAttributeCommand.OTHER));
                        break;
                    case 1:
                        model = new CardModel("Absorb Stones", 10, getCardModelType(), " Destroy +10 Stones ",
                            new ChangeAttributeCommand("Stones", -10, ChangeAttributeCommand.OTHER));
                        break;
                    case 2:
                        model = new CardModel("Absorb Armies", 10, getCardModelType(), " Destroy +10 Monsters ",
                            new ChangeAttributeCommand("Monsters", -10, ChangeAttributeCommand.OTHER));
                        break;
                    case 3:
                        model = new CardModel("Meditation", 20, getCardModelType(), " Attack +12 Enemy Tower ",
                            new ChangeAttributeCommand("Tower", -12, ChangeAttributeCommand.OTHER));
                        break;
                    case 4:
                        model = new CardModel("Wicked Cave", 15, getCardModelType(), " Acquire +30 Stones",
                            new ChangeAttributeCommand("Stones", 30, ChangeAttributeCommand.CURRENT));
                        break;
                    case 5:
                        model = new CardModel("Magic Jar", 15, getCardModelType(), " Acquire +30 Monsters",
                            new ChangeAttributeCommand("Monsters", 30, ChangeAttributeCommand.CURRENT));
                        break;
                    case 6:
                        model = new CardModel("Implosion", 30, getCardModelType(), " Attack +2 Enemy Gem Prod. ",
                            new ChangeAttributeCommand("GemProduction", -2, ChangeAttributeCommand.OTHER));
                        break;
                    case 7:
                        model = new CardModel("Shadow Seer", 30, getCardModelType(), " Attack +2 Enemy Monst. Prod. ",
                            new ChangeAttributeCommand("MonsterProduction", -2, ChangeAttributeCommand.OTHER));
                        break;
                    case 8:
                        model = new CardModel("Unicorn Vision", 30, getCardModelType(), " Attack +2 Enemy Stone Prod. ",
                            new ChangeAttributeCommand("StoneProduction", -2, ChangeAttributeCommand.OTHER));
                        break;
                    case 9:
                        model = new CardModel("Stained Glass", 40, getCardModelType(), " Build +25 Tower ",
                            new ChangeAttributeCommand("Tower", 25, ChangeAttributeCommand.CURRENT));
                        break;
                }
                break;
        }
        return model;
    }
}
