package com.azt.itower.game;

/**
 *
 * As a subclass of Player, ComputerPlayer overides the play method and
 * executes the strategy of playing
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */

import com.azt.itower.util.*;
import com.azt.itower.command.*;

public class ComputerPlayer extends Player {

    /**
     * A Computer Player needs a name
     * @param String The Name of the player
     *
     */
    public ComputerPlayer(String s) {
        super(s);
    }

    /**
     * Waits for the opponent to play
     */
    public void waitToPlay() {
     // pause(20);
    }


    /**
     * promts user for return to indicate the end of a turn
     */
    public void promptUserForReturn() {
        System.out.println("Press RETURN to allow the Computer to play");
        try {
            System.in.read();
        } catch (Exception ex) {
        }
    }

    private final static int kSTRATEGY_DEFENSE = 0;
    private final static int kSTRATEGY_OFFENSE = 1;
    private final static int kSTRATEGY_BUILDUP = 2;
    private final static int kSTRATEGY_RANDOM = 3;
    private static int CurrentStrategy = 0;
    private int fDifOffset = 30;

    private void selectStrategy() {
       Player otherPlayer = IvoryTower.otherPlayer;
       if (otherPlayer.getTowerSize()>137-fDifOffset) {
          CurrentStrategy = kSTRATEGY_OFFENSE;
          return;
       }
       if (getTowerSize()<fDifOffset) {
          CurrentStrategy = kSTRATEGY_DEFENSE;
          return;
       }
       if (getTowerSize()+fDifOffset>otherPlayer.getTowerSize()){
          CurrentStrategy = kSTRATEGY_BUILDUP;
          return;
       }
       else
          CurrentStrategy = kSTRATEGY_BUILDUP;


    }

    private void evaluateCards(Card[] cards) {
       int weight = 0;
       Player otherPlayer = IvoryTower.otherPlayer;

       for (int i = 0;i<cards.length ;i++ ) {
          weight = 0;

          CardModel cm = cards[i].getModel();
          Command cmd = cm.getCommand();



          if (cmd instanceof ChangeAttributeCommand) {
             ChangeAttributeCommand cac = (ChangeAttributeCommand)cmd;
             String cardType = cac.getType();
             int cardChange = cac.getChange();
             int cardTarget = cac.getTarget();


             if (cardTarget == ChangeAttributeCommand.CURRENT) {
                //////////////////////////////////////
                // It's a defensive or build up card
                //////////////////////////////////////
                if (cardType.equals("Tower")) {
                   cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange));
                   if  (CurrentStrategy == kSTRATEGY_DEFENSE)
                    cards[i].setWeight(cards[i].getWeight()*2);

                }
                else if (cardType.equals("Wall")) {
                   cards[i].setWeight(cards[i].getWeight()+Math.abs((int)(cardChange/2)));
                    if  (CurrentStrategy == kSTRATEGY_DEFENSE)
                    cards[i].setWeight(cards[i].getWeight()*2);
                    if (getWallSize()<fDifOffset/3)
                    cards[i].setWeight(cards[i].getWeight()*3);
                }
                else  if (cardType.equals("Gems")) {
                   if (getGems()<fDifOffset/3) {
                     cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*2));
                   }
                   if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                   if (getGems()>fDifOffset*2)
                      cards[i].setWeight(0);

                }
                else if (cardType.equals("Monsters")) {
                   if (getMonsters()<fDifOffset/3) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*2));
                   }
                      if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                      if (getMonsters()>fDifOffset*2)
                      cards[i].setWeight(0);
                }
                else if (cardType.equals("Stones")) {
                 if (getStones()<fDifOffset/3) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*2));
                 }
                     if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                     if (getStones()>fDifOffset*2)
                      cards[i].setWeight(0);

                }
                else  if (cardType.equals("GemProduction")) {
                   if (getGemProduction()<fDifOffset/7) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                   }
                  if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                  if (getGemProduction()>fDifOffset/4)
                      cards[i].setWeight(0);
                  if ( (getGemProduction()<getStoneProduction()) ||
                       (getGemProduction()<getMonsterProduction()) ) {
                       cards[i].setWeight(cards[i].getWeight()*2);
                       }

                }
                else if (cardType.equals("MonsterProduction")) {
                   if (getMonsterProduction()<fDifOffset/7) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                   }
                   if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                   if (getMonsterProduction()>fDifOffset/4)
                      cards[i].setWeight(0);
                   if ( (getMonsterProduction()<getStoneProduction()) ||
                       (getMonsterProduction()<getGemProduction()) ) {
                       cards[i].setWeight(cards[i].getWeight()*2);
                       }


                }
                else if (cardType.equals("StoneProduction")) {
                 if (getStoneProduction()<fDifOffset/7) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                 }
                 if  (CurrentStrategy == kSTRATEGY_BUILDUP)
                      cards[i].setWeight(cards[i].getWeight()*2);
                 if (getStoneProduction()>fDifOffset/4)
                      cards[i].setWeight(0);
                  if ( (getStoneProduction()<getGemProduction()) ||
                       (getStoneProduction()<getMonsterProduction()) ) {
                       cards[i].setWeight(cards[i].getWeight()*2);
                       }


                }

             } else if (cardTarget == ChangeAttributeCommand.OTHER) {

                //////////////////////////////////////
                //it's an offensive card
                //////////////////////////////////////
                if (cardType.equals("Tower")) {
                   if (otherPlayer.getTowerSize()<=Math.abs(cardChange)) {
                   cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*100));
                   }
                   else if (otherPlayer.getTowerSize()+(int)(fDifOffset/2)<=Math.abs(cardChange)) {
                   cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*10));
                   }
                   cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange));
                   if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                    cards[i].setWeight(cards[i].getWeight()*2);

                }
                else if (cardType.equals("Wall")) {
                   cards[i].setWeight(cards[i].getWeight()+Math.abs((int)(cardChange/2)));
                    if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                    cards[i].setWeight(cards[i].getWeight()*2);
                    if (otherPlayer.getWallSize()<fDifOffset/3)
                    cards[i].setWeight(cards[i].getWeight()*4);

                }
                else  if (cardType.equals("Gems")) {
                   if ( (otherPlayer.getGems() > Math.abs(cardChange) ) &&
                   (otherPlayer.getGems()<cardChange*3)) {
                     cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*3));
                   }
                   if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight(cards[i].getWeight());

                }
                else if (cardType.equals("Monsters")) {
                   if ( (otherPlayer.getMonsters() > Math.abs(cardChange) ) &&
                   (otherPlayer.getMonsters()<cardChange*3)) {
                     cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*3));
                   }
                   if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight(cards[i].getWeight());

                }
                else if (cardType.equals("Stones")) {
                   if ( (otherPlayer.getStones() > Math.abs(cardChange) ) &&
                   (otherPlayer.getStones()<cardChange*3)) {
                     cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*3));
                   }
                   if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight(cards[i].getWeight());


                }
                else  if (cardType.equals("GemProduction")) {
                   if ( (otherPlayer.getGemProduction()<fDifOffset/7) && (otherPlayer.getGemProduction()>=2) ) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                   } else {
                      if ((otherPlayer.getGemProduction()>=2))
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*7));
                   }
                  if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight((int)cards[i].getWeight()/2);

                }
                else if (cardType.equals("MonsterProduction")) {
                   if ((otherPlayer.getMonsterProduction()<fDifOffset/7) && (otherPlayer.getMonsterProduction()>=2)) {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                   } else {
                      if ((otherPlayer.getMonsterProduction()>=2))
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*7));
                   }
                  if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight((int)cards[i].getWeight()/2);

                }
                else if (cardType.equals("StoneProduction")) {
                 if ((otherPlayer.getStoneProduction()<fDifOffset/7) && (otherPlayer.getStoneProduction()>=2))  {
                      cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*20));
                 } else {
                     if ((otherPlayer.getStoneProduction()>=2))
                     cards[i].setWeight(cards[i].getWeight()+Math.abs(cardChange*7));
                 }
                  if  (CurrentStrategy == kSTRATEGY_OFFENSE)
                      cards[i].setWeight((int)cards[i].getWeight()/2);

                }
             }
             else if (cardTarget == ChangeAttributeCommand.BOTH) {
                //it's a special card
             }
          }
        }

    }

    /**
     * Selects and plays a Card from the current deck
     */
    public void playCard() {
        int mx_cost = 0;
        int indx = 0;
        Card swapTemp;
        Card[] cards = getHand().getCards();
        //promptUserForReturn();
        // The Computer first sorts its hand
        // in order of preference (i.e. prefers to
        // play the card that cost the least

        //System.out.println("==========================================");
        //System.out.println("======== Computer starts playing =========");



        selectStrategy();
        /*System.out.print  ("======== Strategy selected :: ");

        switch(CurrentStrategy) {
           case kSTRATEGY_DEFENSE : System.out.println("Defense");break;
           case kSTRATEGY_OFFENSE : System.out.println("Offense");break;
           case kSTRATEGY_BUILDUP : System.out.println("BuildUp");break;
           case kSTRATEGY_RANDOM : System.out.println("Random");break;
        }
        */
        evaluateCards(cards);



        pause(1000);
        for (int i = 0; i < cards.length; i++) {
            for (int j = 0; j < cards.length - 1; j++) {

                if (cards[j].getWeight() < cards[j + 1].getWeight()) {
                    swapTemp = cards[j];
                    cards[j] = cards[j + 1];
                    cards[j + 1] = swapTemp;
                }
            }
        }
/*
        System.out.println("======== Weighted Computer Hand =========");
        for (int i = 0; i < cards.length; i++) {
          System.out.println(cards[i].getName() + " has weight:" + cards[i].getWeight() + ":" + cards[i].canExecute() + " : " + cards[i].getModel().getActionTitle());

        }
*/
        // Now the computer player has to select and execute a card
        //
        boolean card_executed = false;
        Card tempCard = null;
        for (int i = 0; i < getHand().getLength(); i++) {
            tempCard = getHand().getCardAt(i);
            if (tempCard.canExecute()) {
                card_executed = true;
            }
            if (card_executed) break;
        }
        // If there is no executable card then we need a strategy to discard
        if (!card_executed) {
            for (int i = 0; i < getHand().getLength(); i++) {
                Card tCard = getHand().getCardAt(i);
                if (tempCard.getWeight() > tCard.getWeight())
                    tempCard = tCard;
                tempCard.setDiscarded(true);
            }
        } else {
            tempCard.execute();
        }
        //GameLogger.log("Computer plays " + tempCard.getName() + " that costs  " + tempCard.getCost());
        // + " and weights " + tempCard.getWeight());
        updateViews();
        pause(200);
        DeckManager.getInstance().pushCard(tempCard);
        for (int i = 0; i < cards.length; i++) {
          cards[i].setWeight(0);
        }
        getHand().clearCard(tempCard);
        updateDecks();
        return;

    }
}
