package com.azt.itower.game;

import java.awt.*;
import java.util.*;
import com.azt.itower.util.*;
import com.azt.itower.deck.*;

/**
 *
 * A DeckManager manages the playing cards
 * It can schuffle the cards and pop and push to any of its two stacks - the used cards stack and the used cards stack
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public class DeckManager implements ResourceConstants {
    private Stack unusedDeck;
    private Stack usedDeck;
    private static DeckManager fgDeckManager = new DeckManager();
    private Image grayDeck;
    private Image greenDeck;
    private Image blueDeck;

    /**
     * Only the Deckanager can create an instance of itself
     */
    private DeckManager() {
        usedDeck = new Stack();
        unusedDeck = new Stack();
        fillUnusedDeck();
    }

    /**
     * Fills the deck using cards out of specific factories
     * This information should normally be read from a configuration
     * file. Specially if a tool to create decks is built.
     */
    private void fillUnusedDeck() {
        grayDeck = IvoryTower.loadImage(kGRAY_DECK_IMG);
        blueDeck = IvoryTower.loadImage(kPURPLE_DECK_IMG);
        greenDeck = IvoryTower.loadImage(kGREEN_DECK_IMG);
        try {
            (AbstractDeckFactory.getFactory("com.azt.itower.deck.GrayDeckFactory")).addCards(unusedDeck);
            (AbstractDeckFactory.getFactory("com.azt.itower.deck.GreenDeckFactory")).addCards(unusedDeck);
            (AbstractDeckFactory.getFactory("com.azt.itower.deck.PurpleDeckFactory")).addCards(unusedDeck);
        }
        catch (GameException ex) {
            GameLogger.log(ex);
        }
    }

    /**
     * Singleton method to return the one and only instance of the Deck Manager\
     * @return DeckManager The single instance of DeckManager
     */
    public static DeckManager getInstance() {
        return fgDeckManager;
    }

    /**
     * Shuffles the deck
     * The way the shuffling happens is a good candidate for a Shuffling Strategy object
     */
    public void shuffle() {
        //GameLogger.log("Shuffling===========================");
        Random myRandom = new Random();
        Stack shuffledStack = new Stack();
        while (!usedDeck.empty()) {
          unusedDeck.push((Card)usedDeck.pop());
        }
        Card[] tempArray = new Card[120];
        for (Enumeration e = unusedDeck.elements(); e.hasMoreElements(); ) {
            Card tempCard = (Card)e.nextElement();
            tempCard.reset();
            int random_index = (int)(myRandom.nextFloat() * 120);
            if (tempArray[random_index] == null)
                tempArray[random_index] = tempCard;
            else {
                for (int k = 0; k < tempArray.length; k++) {
                    if (tempArray[k] == null) {
                        tempArray[k] = tempCard;
                        k = tempArray.length;
                    }
                }
            } // end else
        } //end for
        for (int k = 0; k < tempArray.length; k++) {
            if (tempArray[k] != null)
                shuffledStack.push(tempArray[k]);
        }
        unusedDeck = shuffledStack;
    }

    /**
     * Prints the deck for debugging purposes
     */
    public void printDeck() {
        GameLogger.log("==============================");
        for (Enumeration e = unusedDeck.elements(); e.hasMoreElements(); ) {
            GameLogger.log(((Card)e.nextElement()).getName());
        }
        GameLogger.log("==============================");
    }

    /**
     * Pops a card out of the unused stack
     * If the unused stack finishes then it refills from the used stack
     * and it reshuffles
     * @return Card Just looks at the Card on top of the Stack.
     * @see java.util.Stack
     */
    public Card popCard() {
        if (unusedDeck.empty()) {
            for (Enumeration e = usedDeck.elements(); e.hasMoreElements(); ) {
                unusedDeck.addElement(e.nextElement());
            }
            usedDeck = new Stack();
            shuffle();
        }
        Card tempCard = (Card)unusedDeck.pop();
        //System.out.println("Popping Card" +tempCard.getName());
        return tempCard;
    }

    /**
     * Peeks a Card
     * @return Card Gets the Card on top of the Stack.
     * @see java.util.Stack
     */
    public Card peekCard() {
        if (usedDeck.empty()) {
            return null;
        } else
            return (Card)usedDeck.peek();
    }


     /**
     * Peeks at two Card
     * @return Card[] Gets the two Cards on top of the Stack.
     * @see java.util.Stack
     */
    public Card[] peekTwoCards() {
        Card[] c = new Card[2];
        c[0] = null;
        c[1] = null;
        if (usedDeck.empty()) {
            return c;
        } else {
           Card tempCard = (Card)usedDeck.pop();
           if (usedDeck.empty()) {
              usedDeck.push(tempCard);
              c[0] = tempCard;
              return c;
           } else {
             Card secondCard = (Card)usedDeck.peek();
             c[0] = secondCard;
             c[1] = tempCard;
             usedDeck.push(tempCard);
           }

        }
        return c;
    }

    /**
     * Pushes a Card to the used Deck
     * @param Card Adds a Card to the Stack
     * @see java.util.Stack
     */
    public void pushCard(Card c) {
        usedDeck.push(c);
    }
}
