package com.azt.itower.game;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import com.azt.itower.util.*;

/**
 * This executable class starts off the game
 * It runs both as an applet and as an application
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public class IvoryTower extends Applet implements ResourceConstants, KeyListener {

    static boolean runAsApplication = false;
    Player fPlayer1;Player fPlayer2;
    IvoryTowerView fIvoryTowerView;
    CastlePanel fCastlePanel;
    DeckPanel fDeck1Panel;DeckPanel fDeck2Panel;
    static IvoryTower thisApplet;
    public static Player currentPlayer;
    public static Player otherPlayer;
    private GameCondition currentGameCondition;
    private boolean EndGame = false;
    public static final int kGAME_STATE_INTRO = 0;
    public static final int kGAME_STATE_PLAYING = 1;
    public static final int kGAME_STATE_LOOSE = 2;
    public static final int kGAME_STATE_WIN = 3;
    public static final int kGAME_STATE_STARTING = 4;
    public static int GameState = kGAME_STATE_INTRO;


    /**
     * Applet Entry method init. Inititalises the applet
     */
    public void init() {
        thisApplet = this;
        //GameLogger.log("init called");
        int selector;
        if (!runAsApplication)
            SoundRegistry.getInstance().registerApplet(this);
        try {
            SoundRegistry.getInstance().loadAudioClip(kPICK_SND);
            SoundRegistry.getInstance().loadAudioClip(kRELEASE_SND);
            SoundRegistry.getInstance().loadAudioClip(kSHUFFLE_SND);
            SoundRegistry.getInstance().loadAudioClip(kWIN_SND);
            SoundRegistry.getInstance().loadAudioClip(kLOSE_SND);

            SoundRegistry.getInstance().loadAudioClip(kINTRO_SND);

            SoundRegistry.getInstance().loadAudioClip(kBUILDWALL_SND);
            SoundRegistry.getInstance().loadAudioClip(kBUILDCASTLE_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYWALL_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYCASTLE_SND);

            SoundRegistry.getInstance().loadAudioClip(kRAISESTONES_SND);
            SoundRegistry.getInstance().loadAudioClip(kRAISEGEMS_SND);
            SoundRegistry.getInstance().loadAudioClip(kRAISEMONSTERS_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYSTONES_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYGEMS_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYMONSTERS_SND);

            SoundRegistry.getInstance().loadAudioClip(kRAISESTONEPROD_SND);
            SoundRegistry.getInstance().loadAudioClip(kRAISEGEMPROD_SND);
            SoundRegistry.getInstance().loadAudioClip(kRAISEMONSTERPROD_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYSTONEPROD_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYGEMPROD_SND);
            SoundRegistry.getInstance().loadAudioClip(kDESTROYMONSTERPROD_SND);

            initGame();
            initGraphics();
            registerListeners();
            addKeyListener(this);
            requestFocus();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Overides and sets the Traversable property to true
     * This is needed to be able and read the key S or s to start the game
     * @return boolean
     */
    public boolean isFocusTraversable() {
        return true;
    }

     /**
      * Applet Entry method start. Starts the applet
      */
     public void start() {
        //SoundRegistry.getInstance().playWavAudio(kWIN_SND);
        //GameLogger.log("start called");
        while (!EndGame) {
            intro();
            initGame();
            registerListeners();
            playGame();
        }
    }

   /**
    * Applet Entry method start. Starts the applet
    */
    public void intro() {
        GameState = kGAME_STATE_INTRO;
        fCastlePanel.playIntro();
        while (true) {
            pause(500);
            requestFocus();
            if (GameState == kGAME_STATE_PLAYING) break;
        }
    }

    /**
     * Empty implementation to satisfy the KeyListener interface
     * @param KeyEvent
     * @see java.awt.event.KeyEvent
     */
    public void keyTyped(KeyEvent e) {
    }

    /**
     * Empty implementation to satisfy the KeyListener interface
     * @param KeyEvent
     * @see java.awt.event.KeyEvent
     */
    public void keyPressed(KeyEvent e) {
    }

    /**
     * Handles the KeyEvents on the release of a key
     * @param KeyEvent
     * @see java.awt.event.KeyEvent
     */
    public void keyReleased(KeyEvent e) {
        if ((e.getKeyChar() == 's') || (e.getKeyChar() == 'S')) {
            if (GameState == kGAME_STATE_INTRO) {
                GameState = kGAME_STATE_STARTING;
                GameState = kGAME_STATE_PLAYING;
            }
        }
    }

    /**
     * Stops the game
     */
    public void stop() {
        //GameLogger.log("stop called");
    }

    /**
     * Destroys the game
     */
    public void destroy() {
        //GameLogger.log("destroy called");
    }

    /**
     * Initialises the game
     */
    public void initGame() {
        currentGameCondition = new StandardGameCondition();
        if (fPlayer1!=null) {
           Hand h = fPlayer1.getHand();
           h.resetHand();
        }
        if (fPlayer2!=null) {
           Hand h = fPlayer2.getHand();
           h.resetHand();
        }

        fPlayer1 = new HumanPlayer("Player 1");
        fPlayer2 = new ComputerPlayer("Computer");
        currentPlayer = fPlayer1;
        otherPlayer = fPlayer2;
        DeckManager dm = DeckManager.getInstance();
        dm.shuffle();
        fPlayer1.setTowerSize(40);
        fPlayer2.setTowerSize(40);
        fPlayer1.setWallSize(10);
        fPlayer2.setWallSize(10);
    }

    /**
     * Registers the listeners of the other panels
     */
    public void registerListeners() {
        fCastlePanel.setPlayers(fPlayer1, fPlayer2);
        fDeck1Panel.setPlayer(fPlayer1);
        fDeck2Panel.setPlayer(fPlayer2);
        fPlayer1.addGameChangeListener(fCastlePanel);
        fPlayer2.addGameChangeListener(fCastlePanel);
        fPlayer1.addDeckChangeListener(fDeck1Panel);
    }

    /**
     * Initialises the Graphics and GUI
     */
    public void initGraphics() {

        setLayout(
            new GridLayout(1, 1, 0, 0));
        fIvoryTowerView = new IvoryTowerView();
        fIvoryTowerView.setLayout(
            new BorderLayout());
        fCastlePanel = new CastlePanel(fPlayer1, fPlayer2);
        fDeck1Panel = new DeckPanel(fPlayer1);
        fDeck2Panel = new DeckPanel(fPlayer2);
        fIvoryTowerView.add("Center", fCastlePanel);
        fIvoryTowerView.add("South", fDeck1Panel);
        add(fIvoryTowerView);
        if (!runAsApplication) {
            setVisible(true);
        }
    }

    /**
     * Main playing loop
     */
    public void playGame() {
    SoundRegistry.getInstance().playWavAudio(kINTRO_SND);
        while (!gameOver()) {
            currentPlayer = fPlayer1;
            otherPlayer = fPlayer2;
            fDeck1Panel.acceptInteraction(true);
            fCastlePanel.repaint();
            fPlayer1.play();
            fDeck1Panel.acceptInteraction(false);
            pause(10);
            if (gameOver()) break;
            currentPlayer = fPlayer2;
            otherPlayer = fPlayer1;
            fCastlePanel.repaint();
            fPlayer2.play();
            pause(100);
        }
        if (fPlayer1.wins(currentGameCondition) || fPlayer2.loses(currentGameCondition)) {
            //Player 1 you have won!
            //GameLogger.log(fPlayer1.getName() + " you win...");
            SoundRegistry.getInstance().playWavAudio(kWIN_SND);
            GameState = kGAME_STATE_WIN;
            fCastlePanel.repaint();
        }
        if (fPlayer2.wins(currentGameCondition) || fPlayer1.loses(currentGameCondition)) {
            //Player 1 you have lost!
            //GameLogger.log(fPlayer1.getName() + " you loose...");
            SoundRegistry.getInstance().playWavAudio(kLOSE_SND);
            GameState = kGAME_STATE_LOOSE;
            fCastlePanel.repaint();
        }
        pause(5000);
        //reset player's hand

        GameState = kGAME_STATE_INTRO;
        fCastlePanel.repaint();
    }

    /**
     * Pauses the execution of the currently executing thread
     * @param int The amount of time in Milliseconds
     */
    public void pause(int mil) {
        try {
            Thread.currentThread().sleep(mil);
        } catch (Exception ex) {
        }
    }

    /**
     * Chacks game over condition
     * @return boolean Returns the result from checking the game over condition
     */
    public boolean gameOver() {
        return ((fPlayer1.loses(currentGameCondition)) || (fPlayer2.loses(currentGameCondition)) ||
            (fPlayer1.wins(currentGameCondition)) || (fPlayer2.wins(currentGameCondition)));
        //((fPlayer1.getTowerSize()<0 || fPlayer2.getTowerSize()<0) || (fPlayer1.getTowerSize()>100 || fPlayer2.getTowerSize()>100) ) ;
    }

    /**
     * Runs as an application that creates a frame and puts the applet
     */
    public static void main(String[] argv) {
        runAsApplication = true;
        final Frame f = new Frame("IvoryTower v0.0 azt'2000");
        IvoryTower jam = new IvoryTower();
        f.setLayout(
            new GridLayout(1, 1, 0, 0));
        f.add(jam);
        f.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    f.dispose();
                    System.exit(0);
                }
            });
        jam.init();
        f.pack();
        f.setVisible(true);
        f.setResizable(false);
        jam.start();
    }

    /**
     * Utility method to return an image by name
     * @param String The FileName of the BackgroundImage
     * @return Image
     * @see java.awt.Image
     */
    public static Image loadImage(String relName) {
        if (runAsApplication) {
            MediaTracker tracker = new MediaTracker(thisApplet);
            Image imtexture = Toolkit.getDefaultToolkit().getImage(relName);
            tracker.addImage(imtexture, 0);
            try {
                tracker.waitForID(0);
                int width = imtexture.getWidth(thisApplet);
                int height = imtexture.getHeight(thisApplet);
            }
            catch (Exception e) {
                GameLogger.log("Exception on Image-Texture Loading", e);
            }
            return imtexture;
        }
        else {
            return (thisApplet.getImage(thisApplet.getCodeBase(), relName));
        }
    }

    /**
     * Returns an absolute dimension as a size
     * @return Dimension
     * @see java.awt.Component.getPreferredSize()
     */
    public Dimension getPreferredSize() {
        return new Dimension(780, 600);
    }
}
