package com.azt.itower.util;

import java.net.*;
import java.applet.*;
import java.util.*;
import com.azt.itower.game.GameException;

/**
 * Title: The Ivory Tower Card Game <p> Description:  A first attempt at a fantasy card game!! <p> Copyright feb2000 AZT  Copyright (c) <p>
 * A Singleton objects that loads and caches sound files
 * @author thanassis@thanassis.com
 * @version 0.9 beta
 */
public class SoundRegistry {
    private static SoundRegistry fgSoundRegistry;
    private Hashtable fMap;
    private Applet applet = null;

    /**
     * Only SoundRegistry itself can make instances of itslef
     */
    private SoundRegistry() {
        fMap = new Hashtable();
    }

    /**
     * Register Applet requesting sound files
     * @param Applet The applet that needs the sounds
     * (needed only if we rub as an applet
     */
    public void registerApplet(Applet app) {
        applet = app;
    }

    /**
     * Returns an instance of the SoundRegistry
     * @return SoundRegistry The single instance of the SoundRegistry
     */
    public static SoundRegistry getInstance() {
        if (fgSoundRegistry == null) {
            synchronized(SoundRegistry.class) {
                if (fgSoundRegistry == null) {
                    fgSoundRegistry = new SoundRegistry();
                }
            }
        }
        return fgSoundRegistry;
    }


    /**
     * Loads an audio file with the given name.
     * @param String The absolute filepath of the soundfile
     * @return Audioclip The Audioclip for the provided name
     * @exception GameException Raised if anything goes wrong on the loading
     * of the soundfile
     */
    public AudioClip loadAudioClip(String absfilename) throws GameException {
        if (absfilename == null)
            throw new GameException("supplied audioclip file was null");
        AudioClip aclip = (AudioClip)fMap.get(absfilename);
        if (aclip == null) {
            aclip = loadAudioResource(absfilename);
            if (aclip != null)
                fMap.put(absfilename, aclip);
        }
        return aclip;
    }

    /**
     * Loads an audio clip with the given name.
     * @param String The name of the soundfile
     * @return Audioclip The Audioclip for the provided name
     * @exception GameException Raised if anything goes wrong on the loading
     * of the soundfile
     */
    public AudioClip loadAudioResource(String name) throws GameException {
        //GameLogger.log( "Trying to get :" + name );
        AudioClip clip = null;
        if (name == null)
            throw new GameException("supplied audioclip resource was null");


        URL url = null;
        if (applet!=null) {
           url = getClass().getResource(name);
        }

        else {
           try {
             String wd = "file:///" + System.getProperty("user.dir") + java.io.File.separator + name;
             url = new URL(wd);
           }
           catch(Exception ex) {
              throw new GameException(ex.getMessage());
           }
        }

        if (applet == null) // we run as an Application
        {
            if (url == null) {
                GameLogger.log("Null URL");
                //GameLogger.log( "Trying to get clip:" + url.toString() );
            } else
                clip = Applet.newAudioClip(url);
            if (clip == null)
                throw new GameException("unable to load audioclip -> " + name);
        } else if (applet != null) { //we run as an applet
            clip = applet.getAudioClip(applet.getDocumentBase(), name);
        }
        return clip;
    }

    /**
     * Play a Audio clip
     * @param String The name of the sound to play
     */
    public void playWavAudio(String name) {
        try {
            loadAudioClip(name).play();
        }
        catch (GameException gex) {
            GameLogger.log(gex);
        }
    }
}
